<?php

namespace JimsCustomerBooking\Database;

use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Services\DivisionService;

class Booking extends Model
{
    protected $table_name = 'customer_bookings';

    /**
     * Returns the ID of the local booking
     *
     * @param $data
     * @param $status
     * @return int
     */
    public function add($data, $status = 'pending')
    {
        $title = $this->calculateTitleFromData($data);

        return $this->insert([
            'title' => $title,
            'booking_id' => md5(time() . $title),
            'data' => \json_encode($data),
            'status' => $status,
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s'),
        ]);
    }

    protected function calculateTitleFromData($data)
    {
        $divisionService = new DivisionService();
        $services = $divisionService->getServices(true);
        $first_service = $data['services'][0];

        $service = array_filter($services, function ($item) use ($first_service) {
            return $item['id'] == $first_service;
        });

        $service = array_shift($service);

        return $service['name'];
    }

    /**
     * @param $page
     * @param $limit
     * @param $status
     * @return array|null
     */
    public function getPage($page = 1, $limit = 25, $status = 'all')
    {
        $offset = ($page - 1) * $limit;
        $table = $this->getTable($this->table_name);

        if ($status == 'all') {
            $sql = "SELECT * FROM $table ORDER BY created_at ASC LIMIT %d OFFSET %d";
            $sql = $this->prepare($sql, [$limit, $offset]);
        } else {
            $sql = "SELECT * FROM $table WHERE booking_status = %s ORDER BY created_at ASC LIMIT %d OFFSET %d";
            $sql = $this->prepare($sql, [$status, $limit, $offset]);
        }

        //apply filter
        $sql = apply_filters(StringHelper::GetPrefixed('query_booking_get_page'), $sql, $page, $limit, $status);

        return $this->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    /**
     * @param $limit
     * @param $status
     * @return int
     */
    public static function getPageCount($limit = 10, $status = 'all')
    {
        $self = new static;
        $table = $self->getTable();

        if ($status == 'all') {
            $sql = "SELECT COUNT(*) FROM $table";
        } else {
            $sql = "SELECT COUNT(*) FROM $table WHERE status = %s";
            $sql = $self->prepare($sql, [$status]);
        }

        //apply filter
        $sql = apply_filters(StringHelper::GetPrefixed('query_booking_get_page_count'), $sql, $limit, $status);

        $all_records_count = $self->get_var($sql);
        $pages = ceil($all_records_count / $limit);

        return $pages;
    }

    public static function getStatisticsTotals($status = 'sended')
    {
        $self = new static;
        $table = $self->getTable();

        $sql = "SELECT title, COUNT(*) AS total_bookings, SUM(CASE WHEN status = %s THEN 1 ELSE 0 END) AS total_bookings_with_status FROM $table GROUP BY title ORDER BY title, created_at;";
        $sql = $self->prepare($sql, [$status]);

        return $self->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    public static function getStatisticsTotalsByDate($date = null, $status = 'sended')
    {
        $self = new static;
        $table = $self->getTable();

        if ($date === null) {
            $date = date('d/m/Y');
        }

        $sql = "SELECT title, COUNT(*) AS total_bookings, SUM(CASE WHEN status = '$status' THEN 1 ELSE 0 END) AS total_bookings_with_status FROM $table WHERE DATE(created_at) = STR_TO_DATE('$date', '%d/%m/%Y') GROUP BY title ORDER BY title, created_at;";
//        $sql = $self->prepare($sql, [$status, $date]);

        return $self->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }
}