<?php

namespace JimsCustomerBooking\Database;

use JimsCustomerBooking\Jims_Plugin;

class Database
{
    public static function setupOrUpdateDatabase()
    {
        self::setup();
    }

    /**
     * @return void
     */
    public static function setup():void {
        self::createTable([
            'id mediumint(9) NOT NULL AUTO_INCREMENT',
            'booking_id varchar(255) NOT NULL',
            'title varchar(255) NULL',
            'data longtext NOT NULL',
            'status varchar(255) NOT NULL',
            'created_at datetime DEFAULT \'0000-00-00 00:00:00\' NOT NULL',
            'updated_at datetime DEFAULT \'0000-00-00 00:00:00\' NOT NULL',
            'PRIMARY KEY  (id)'
        ], 'customer_bookings');
    }


    /**
     * Creates a table in the database, from an array of columns
     *
     * @param array $columns
     * @param $table_name
     * @return array|bool
     */
    public static function createTable(array $columns, $table_name){
        global $wpdb;

        if(count($columns) == 0){
            return false;
        }

        $charset_collate = $wpdb->get_charset_collate();

        $bookings_table_name = self::getTableName ($wpdb, $table_name);

        $columns = implode(',', $columns);
        $sql = "CREATE TABLE IF NOT EXISTS $bookings_table_name ($columns) $charset_collate;";

        $result = self::runSQL($sql);

        return [
            'table_name' => $bookings_table_name,
            'sql' => $sql,
            'result' => $result
        ];
    }

    public static function deleteAllTransients()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . 'options';
        $plugin_prefix = Jims_Plugin::PREFIX;

        $sql =  "DELETE FROM $table_name WHERE `option_name` like '%transient%' and `option_name` like '%$plugin_prefix%';";

        return $wpdb->query($sql);
    }

    /**
     * @param $sql
     * @return array|false|int
     */
    public static function runSQL($sql)
    {
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        return dbDelta($sql);
    }

    public static function deleteDatabase()
    {
        global $wpdb;

        $table_name = self::getTableName ($wpdb);

        $sql = "DROP TABLE IF EXISTS $table_name";

        $wpdb->query($sql);
    }

    /**
     * @param \wpdb $wpdb
     * @return string
     */
    protected static function getTableName(\wpdb $wpdb, $name = 'customer_bookings'): string
    {
        return implode('_', [$wpdb->prefix . Jims_Plugin::PREFIX, $name]);
    }
}