<?php

namespace JimsCustomerBooking\Helpers;

use JimsCustomerBooking\Jims_Plugin;
use JimsCustomerBooking\Validators\JsonValidator;
use JimsCustomerBooking\Validators\Validator;

if (!class_exists ('JimsCustomerBooking\Helpers\StringHelper')) {
    /**
     *
     */
    class StringHelper extends Helper
    {


        const PREFIXED_TYPE_SLUG = 'slug';

        const PREFIXED_TYPE_MACHINE = 'machine';

        /**
         * @param string $string
         * @param string $type , slug|machine
         * @return string
         */
        public static function GetPrefixed($string, $type = self::PREFIXED_TYPE_SLUG)
        {
            list($prefix, $replacement) = self::_getReplacementValues ($type);

            $result = preg_replace ('#[\-_ ]#', $replacement, "{$prefix} {$string}");
            return is_string ($result) ? strtolower ($result) : strtolower (implode ($replacement, $result));
        }

        public static function getPrefix($type = self::PREFIXED_TYPE_SLUG)
        {
            list($prefix, $replacement) = self::_getReplacementValues ($type);
            return preg_replace ('#[\-_ ]#', $replacement, "{$prefix}");
        }

        /**
         * Same as GetPrefixed but takes an array of parts
         *
         * @param array $parts
         * @param $type
         * @return string
         */
        public static function GetPrefixedFromParts($parts, $type = self::PREFIXED_TYPE_SLUG)
        {
            $string = implode (' ', $parts);
            return self::GetPrefixed ($string, $type);
        }

        /**
         * @param $type
         * @return array
         */
        public static function _getReplacementValues($type)
        {
            return [
                Jims_Plugin::PREFIX,
                [self::PREFIXED_TYPE_SLUG => '-', self::PREFIXED_TYPE_MACHINE => '_'][$type]
            ];
        }

        /**
         * @param $string
         * @return bool
         */
        public static function isJson($string) {
          return JsonValidator::isJson($string);
        }

        /**
         * @param $string
         * @return bool
         */
        public static function isSerialized($string) {
            return (@unserialize($string) !== false);
        }
    }

}