<?php

namespace JimsGroup\Apis;

use JimsGroup\Utilities\HttpHeaders;
use JimsGroup\Utilities\HttpRequest;

//use JimsGroup\Utilities\Logger;

/**
 * Class AbstractApi
 * @package JimsGroup\Apis
 */
abstract class AbstractApi
{

    /**
     * Refers to the api urls keys
     * @var string
     */
    private $mode = 'production';


    /**
     * @var string[] $api_urls
     */
    protected $api_urls = [
        'production' => 'https://api.jims.net',
        'uat' => 'https://api.uat.aws.jims.net', //user acceptance testing
        'hf' => 'https://api.hf.aws.jims.net', // hotfix
        //'hf2' => 'https://api2.hf.aws.jims.net', // hotfix 2
        'development' => 'https://api.int.aws.jims.net',
    ];

    /**
     * @var array $paths
     */
    protected $paths = [];

    public function __construct($mode = 'production')
    {
        $this->mode = $mode;
    }

    /**
     * @param string $name
     * @return string
     */
    public function getEndpointUrl($name)
    {
        return $this->getApiUrl() . $this->paths[$name];
    }

    /**
     * @param string $mode
     */
    public function getApiUrl()
    {
        return $this->api_urls[$this->mode];
    }

    public function getApiUrls()
    {
        return $this->api_urls;
    }

    /**
     * @param mixed|string $mode
     */
    public function setMode(mixed $mode): void
    {
        $this->mode = $mode;
    }


    /**
     * @param $url
     * @param $params
     * @return string
     */
    protected function post($url, $params)
    {
        $request = new HttpRequest();
        $request->setUrl($url);
        $request->setMethod('POST');
        $request->setHeaders([
            'Content-Type: application/json',
            'Accept: application/json',
        ]);
        $request->setBody(json_encode($params));
        $request->send();

        $headers = $request->getResponseHeaders(true);

//        Logger::simpleLog ($headers, 'headers.log', true);

        return $request->getResponse();
    }

    protected function get($url, $params = [], $headers = [])
    {
        $request_headers = new HttpHeaders([
            'Content-Type' =>  'application/json',
            'Accept' => 'application/json',
        ]);

        if (count($headers) > 0) {
            foreach ($headers as $header_name => $header) {
                $request_headers->add($header_name, $header);
            }
        }

        if(count($params) > 0 && !empty($params)){
            $params_query = http_build_query($params);
            $url = $url . '?' . $params_query;
        }

        $request = new HttpRequest();
        $request->setUrl($url);
        $request->setMethod('GET');
        $request->setHeaders($request_headers->toCurlArray());
        $request->send();

        //uncomment to log headers
//        $headers = $request->getResponseHeaders(true);
//        error_log(json_encode($headers), 0);


        return $request->getResponse();
    }
}