<?php

namespace JimsGroup\Datatypes;

abstract class Datatype
{
    protected $default = [];

    protected $data = [];

    /**
     * @var array of keys to ignore when validating
     */
    protected $ignore_keys = [
        'unit_number'
    ];

    public function __construct(array $data = [], $ignore_validation = false)
    {
        if (!$this->validate($data) && !$ignore_validation) {
            throw new \InvalidArgumentException('Invalid data');
        }

        $this->data = array_merge($this->default, $this->data, $data);
    }

    public function __get($name)
    {
        return $this->data[$name] ?? null;
    }

    protected function set($name, $value)
    {
        $this->data[$name] = $value;
    }

    public function toArray()
    {
        return $this->data;
    }

    public function validate(array $data)
    {
        //if $ignore_keys is not empty, remove the keys from $data and $this->default
        if (!empty($this->ignore_keys)) {
           foreach ($this->ignore_keys as $key) {
               unset($data[$key]);
               unset($this->default[$key]);
           }
        }

        return count( array_diff_key ($data, $this->default) ) === 0;
    }
}